@@ -39,6 +39,8 @@
 #define ALLOW_NOESCAPE PACKAGE_DATA_DIR \
             G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "allow_noescape"
 
+#define MAX_WINDOW_SIZE (G_MAXUINT16 / 2)
+
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) gettext(String)
@@ -400,49 +402,29 @@ cb_timeout (gint *timeout)
 	return TRUE;
 }
 
-#if 0
-gint
-geometry_get_number (const gchar **geometry)
+
+gboolean
+label_width_okay (const gchar *str)
 {
-	/* (This function is taken from GNOME's gnome-geometry.c) */
+	GtkWidget *dummy;
+	PangoLayout *layout;
+	int width;
 
-	gint value = 0;
-	gint mult  = 1;
+	dummy = gtk_label_new (NULL);
+	layout = gtk_label_get_layout (GTK_LABEL(dummy));
+	pango_layout_set_text (layout, str, -1);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+	gtk_widget_destroy (dummy);
 
-	if (**geometry == '-') {
-		mult = -1;
-		(*geometry)++;
+	if (width > MAX_WINDOW_SIZE) {
+		g_warning ("%s: button too wide\n", PACKAGE);
+		return FALSE;
 	}
-	while ( **geometry && isdigit(**geometry) ) {
-		value = value * 10 + (**geometry - '0');
-		(*geometry)++;
-	}
-	return value * mult;
+
+	return TRUE;
 }
 
 
-void
-geometry_parse_string (const gchar *geometry, gint *width, gint *height)
-{
-	/* Parse geometry string into width and height */
-
-	*width = *height = -1;
-
-	if (*geometry == '=') geometry++;
-	if (*geometry == '\0') return;
-
-	*width = geometry_get_number (&geometry);
-	if (*width < -1) *width = -1;
-
-	if (*geometry == 'x') {
-		geometry++;
-		if (*geometry == '\0') return;
-		*height = geometry_get_number (&geometry);
-		if (*height < -1) *height = -1;
-	}
-}
-#endif
-
 void
 window_create (void)
 {
@@ -586,12 +568,22 @@ window_create (void)
 					s++;
 				}
 				s = g_strconcat ("gtk-", p, NULL);
+				if (!label_width_okay (s)) {
+					/* XXX: Truncate and carry on, or quit? */
+					prog_cleanup ();
+				}
 				btn = gtk_button_new_from_stock (s);
 				g_free (s);
 				g_free (p);
 			}
 			else {
+
+				if (!label_width_okay (button->label)) {
+					/* XXX: Truncate and carry on, or quit? */
+					prog_cleanup ();
+				}
 				btn = gtk_button_new_with_mnemonic (button->label);
+
 			}
 
 			g_signal_connect (G_OBJECT(btn), "clicked",
@@ -643,24 +635,14 @@ window_create (void)
 		gtk_widget_size_request (btn, &size_req);
 		win_h = win_h + size_req.height + 12;
 	}
-#if 0
-	if (gx.geom_str != NULL) {
-		gint tmp_w, tmp_h;
-		geometry_parse_string (gx.geom_str, &tmp_w, &tmp_h);
-		if (tmp_w != -1) win_w = tmp_w;
-		if (tmp_h != -1) win_h = tmp_h;
-	}
-#endif
 
 	gtk_window_set_position (GTK_WINDOW(window), gx.window_position);
+	gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
 
 	if (gx.geom_str != NULL) {
 		gtk_widget_show_all (vbox); /* must precede parse_geometry */
 		gtk_window_parse_geometry (GTK_WINDOW(window), gx.geom_str);
 	}
-	else {
-		gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
-	}
 
 	/* open the window */
 	gtk_widget_show_all (window);
@@ -692,7 +674,7 @@ read_stdin (void)
 
 
 gchar*
-message_to_utf8 (const gchar *str)
+str_to_utf8 (const gchar *str)
 {
 	gchar *result;
 	GError *error = NULL;
@@ -824,6 +806,7 @@ main (gint argc, gchar *argv[])
 	GString *gstr = NULL;
 	gchar *ch = NULL, *tmpstr;
 	const gchar *fname = NULL;
+	const gchar *bstr = NULL;
 	gint opt, arg = 1;
 	gboolean ok;
 	gchar bu_default[] = "okay:0";
@@ -1011,12 +994,12 @@ main (gint argc, gchar *argv[])
 			g_printerr (_("%s: unable to read file\n"), PACKAGE);
 			prog_cleanup ();
 		}
-		gx.message_text = message_to_utf8 (tmpstr);
+		gx.message_text = str_to_utf8 (tmpstr);
 		gx.message_len = strlen (tmpstr);
 		g_free (tmpstr);
 	}
 	else if (gstr != NULL) {
-		gx.message_text = message_to_utf8 (gstr->str);
+		gx.message_text = str_to_utf8 (gstr->str);
 		gx.message_len = gstr->len;
 		g_string_free (gstr, TRUE);
 	}
